/*
    BSFilter.h  

    Declaration of the BSFilter effect (Bit-Shift Filter).

    ---------------------------------------------------------------------------
 
    Copyright (C) 2016, Cornfield Electronics, Inc.
 
    This work is licensed under the Creative Commons Attribution-ShareAlike 3.0 
    Unported License.
 
    To view a copy of this license, visit
    http://creativecommons.org/licenses/by-sa/3.0/
 
    Created by Bill Alessi & Mitch Altman.
 
   ---------------------------------------------------------------------------
*/

#ifndef BSFILTER_H_INCLUDED
#define BSFILTER_H_INCLUDED

#include "Effect.h"

class BSFilter : public Effect
{
   static const byte MaxClip = 7;   // max # high bits to clip after shift

   int     last;                    // last output value   

   byte    shift;                   // # of bits to (left) shift last value
   byte    mltShift;                // equivalent multiplier for shift 

   byte    numClip;                 // # of high bits to clip after shift
   byte    clipMask;                // bitMask corresponding to numClip

   boolean complement;              // if true, complement the filter output

   public:

   BSFilter(); 

   void   charEv( char );           // process a character event
   void   info();                   // display object info to console
   char   menu( key );              // map key event to character 
   void   process( char* );         // process an input buffer 
   char  *prompt();                 // return object's prompt string
   void   reset();                  // reset object to initial state
   void   setClip( byte );          // set # bits to clip avg
   void   setShift( byte );         // set # bits to (left) shift

   void clipMode( BSFilter* );   // push clip mode

} ;  // BSFilter

#endif   // ifndef BSFILTER_H_INCLUDED
