/*
    Effect.cpp  

    Implementation of the Effect and derived classes.

    ---------------------------------------------------------------------------
 
    Copyright (C) 2016, Cornfield Electronics, Inc.
 
    This work is licensed under the Creative Commons Attribution-ShareAlike 3.0 
    Unported License.
 
    To view a copy of this license, visit
    http://creativecommons.org/licenses/by-sa/3.0/
 
    Created by Bill Alessi & Mitch Altman.
 
   ---------------------------------------------------------------------------
*/

#include "types.h"
#include "Effect.h"
#include "Console_.h"

                        /*--------------------
                              EffectsMode
                         --------------------*/

class EffectsMode : public Mode
{
   public:

   Effect *chain;
   
   void info()
   {
      console.rtab();
      Effect *e = chain;
      while ( e )
      {
         console.print( e->shortcut );
         console.print( ": " );
         console.print( e->prompt() );
         console.space();
         e = e->next;
      }
   }

   void charEv( char code )
   {
      boolean found = false;
      Effect *e = chain;

      if ( code >= '0' && code <= '8' )  // code is a 0-based ordinal
      {
         signed char nth = code - '0';
         while ( e && nth > 0 )
         {
            e = e->next;
            --nth;
         }
         if ( e ) 
         {
            found = true;
            console.pushMode( e );
         }
      }
      else                               // code may be a shortcut letter
      {
         while ( e )
         {
            if ( e->shortcut == code )
            {
               found = true;
               console.pushMode( e );
               break;
            }
            e = e->next;
         }
      }

      if ( ! found ) 
      {
         switch ( code )
         {
            case '.':
            case '<':
            case '!':
            {
               Effect *e = chain;
               while ( e )
               {
                  e->charEv( code );
                  e = e->next;
               }
               break;
            }
            default:
               Mode::charEv( code );
         }
      }

   }

   char menu( key k )
   {
      switch ( k.position() )
      {
         case 11: return '!';
         case 10: return '.';
         case  9: return '<';
         default: return ( '0' + k.position() );
      }
   }

   char *prompt() 
   { 
      return CONSTR("effects"); 
   }

   void reset()
   {
      charEv('!');
   }

} effectsMode;

                        /*--------------------
                              EffectsLoop
                         --------------------*/


EffectsLoop::EffectsLoop()
{
   effects = NULL;
}

void EffectsLoop::append( Effect *e )     // appemd an effect to the effects loop
{
   if ( effects )
   {
      Effect *chain = effects;
      while ( chain->next )
         chain = chain->next;
      chain->next = e;
   }
   else
      effects = e;
}

bool EffectsLoop::_charEv( char code )
{
   if ( code == 'e' )
   {
      effectsMode.chain = effects;
      console.pushMode( &effectsMode );
      return true;                        // "event handled"
   }
   else
      return false;                       // "event not handled"
}

void EffectsLoop::process( char *buf )
{
   Effect *e = effects;
   while ( e )
   {
      if ( ! e->amMute() ) 
         e->process( buf );
      e = e->next;
   }
}

void EffectsLoop::reset()   
{ 
   effectsMode.chain = effects;
   effectsMode.reset();
}

