/*
    Effect.h  

    Declaration of the Effect and derived classes.

    ---------------------------------------------------------------------------
 
    Copyright (C) 2016, Cornfield Electronics, Inc.
 
    This work is licensed under the Creative Commons Attribution-ShareAlike 3.0 
    Unported License.
 
    To view a copy of this license, visit
    http://creativecommons.org/licenses/by-sa/3.0/
 
    Created by Bill Alessi & Mitch Altman.
 
   ---------------------------------------------------------------------------
*/

#ifndef EFFECT_H_INCLUDED
#define EFFECT_H_INCLUDED

#include "Control.h"

/* -------------------------------------------------------------------------- */

class Effect : public Control
{
   public:

   char    shortcut;                // shortcut key to access object's mode
   Effect  *next;                   // ptr to next effect in loop

   virtual void process( char* ){}; // process an input buffer 

} ;  


class EffectsLoop
{
   private:

   Effect* effects;                 // pts to 1st effect in effects loop

   public:

   EffectsLoop();

   void    append( Effect* );       // append an effect to effects loop
   bool   _charEv( char );          // process a character event 
   void    process( char* );        // process audio buffer through effects loop
   void    reset();                 // reset all effects in loop
} ;


/* -------------------------------------------------------------------------- */

class BitShiftFilter : public Effect
{
   static const int MaxClip = 7;    // max # high bits to clip after shift

   int     last;                    // last output value   

   byte    shift;                   // # of bits to (left) shift last value
   byte    mltShift;                // equivalent multiplier for shift 

   byte    numClip;                 // # of high bits to clip after shift
   byte    clipMask;                // bitMask corresponding to numClip

   boolean complement;              // if true, complement the filter output

   public:

   BitShiftFilter(); 

   void  charEv( char );            // process a character event
   void  info();                    // display object info to console
   void  process( char* );          // process an input buffer 
   char *prompt();                  // return object's prompt string
   void  reset();                   // reset object to initial state
   void  setClip( byte );           // set # bits to clip avg
   void  setShift( byte );          // set # bits to (left) shift

   void clipMode( BitShiftFilter* );   // push clip mode

} ;  // BitShiftFilter

#endif   // ifndef EFFECT_H_INCLUDED
