/*
    LFO.h  

    Declaration of the LFO class.

    ---------------------------------------------------------------------------
 
    Copyright (C) 2016, Cornfield Electronics, Inc.
 
    This work is licensed under the Creative Commons Attribution-ShareAlike 3.0 
    Unported License.
 
    To view a copy of this license, visit
    http://creativecommons.org/licenses/by-sa/3.0/
 
    Created by Bill Alessi & Mitch Altman.
 
   ---------------------------------------------------------------------------
*/

#ifndef LFO_H_INCLUDED
#define LFO_H_INCLUDED

#include "Control.h"

class LFO : public Control
{
   public:

   double   val;                 // current output value of oscillator

   virtual void calcRange();     // set top/bot, based on depth
   void   charEv( char );        // process a character event
   void   dynamics();            // update object dynamics
   virtual void evaluate();      // set val, based on pos
   void   info();                // display object info to console
   char   menu( key );           // map key event to character 
   char*  prompt();              // return object's prompt string
   void   reset();               // reset object to initial state
   void   setDepth( double );    // set oscillation depth
   void   setFreq( double );     // set oscillation frequency
   void   setOsc(double,double); // set oscillation depth and freq

   protected:

   double   depth;               // oscillation depth (0.0 - 1.0)

   double   top;                 // top of oscillation range
   double   bot;                 // bot of oscillation range
   double   pos;                 // cur position within oscillation range

   private:

   double   freq;                // oscillation frequency
   double   step;                // change in pos per dynamic update

   void     calcStep();          // calculate step

} ;

#endif   // ifndef LFO_H_INCLUDED
