/*
    Mode.h  

    Declaration of the Mode class.

    ---------------------------------------------------------------------------
 
    Copyright (C) 2016, Cornfield Electronics, Inc.
 
    This work is licensed under the Creative Commons Attribution-ShareAlike 3.0 
    Unported License.
 
    To view a copy of this license, visit
    http://creativecommons.org/licenses/by-sa/3.0/
 
    Created by Bill Alessi & Mitch Altman.
 
   ---------------------------------------------------------------------------
*/

#ifndef MODE_H_INCLUDED
#define MODE_H_INCLUDED

#include "Arduino.h"
#include "but.h"
#include "key.h"

enum focus                             // focus change (as generated by console)
{
   focusPUSH,                          // mode is being pushed on stack
   focusPOP,                           // mode is being popped from stack
   focusRESTORE                        // mode is restored after pop of child
} ;

class Mode
{
   public:

   byte     flags;                     // flag bits denote:

   enum     { ECHO    = 1,             // echo character input to console
              COLON   = 2,             // prompt delimiter is ':' (else '>')
              MENU    = 4,             // charEv(menu()) pre-empts keyEv()
              ONESHOT = 8              // pre-emption is for 1 event only
            };

   Mode();

   virtual void    butEv( but );       // handle a button event
   virtual void    charEv( char );     // handle a character event
   virtual void    info() {};          // display object info to console
   virtual boolean keyEv( key );       // handle a key event
   virtual char    menu( key );        // map key event to character
   virtual void    onFocus( focus ){}; // executed by console on focus change
   virtual char*   prompt();           // return prompt string for this object

} ;

#endif   // ifndef MODE_H_INCLUDED
